@@ -1,7 +1,6 @@
 import unittest
-from htsmodels.models.standard_gp import SGP
 import tsaugmentation as tsag
-import shutil
+from htsmodels.models.standard_gp import SGP
 
 
 class TestModel(unittest.TestCase):
@@ -10,29 +9,23 @@ class TestModel(unittest.TestCase):
         self.data = tsag.preprocessing.PreprocessDatasets('prison').apply_preprocess()
         self.n = self.data['predict']['n']
         self.s = self.data['train']['s']
-        shutil.rmtree("./data/original_datasets")
         self.gpf = SGP('prison', self.data)
 
     def test_results_interval_piecewise(self):
-        model, like = self.gpf.train(n_iterations=500)
-        samples = self.gpf.predict(model, like)
-        res = self.gpf.metrics(samples)
-        self.assertLess(res['mase']['bottom'], 2.5)
+        model, like = self.gpf.train(n_iterations=100)
+        pred, pred_scaled = self.gpf.predict(model, like)
+        res = self.gpf.metrics(pred[0], pred[1])
+        self.assertLess(res['mase']['bottom'], 5)
 
     def test_results_interval_linear(self):
         model, like = self.gpf.train(n_iterations=100, mean_function='linear')
-        samples = self.gpf.predict(model, like)
-        res = self.gpf.metrics(samples)
-        self.assertLess(res['mase']['bottom'], 2.5)
+        pred, pred_scaled = self.gpf.predict(model, like)
+        res = self.gpf.metrics(pred[0], pred[1])
+        self.assertLess(res['mase']['bottom'], 5)
 
     def test_results_interval_zero(self):
         model, like = self.gpf.train(n_iterations=100, mean_function='zero')
-        samples = self.gpf.predict(model, like)
-        res = self.gpf.metrics(samples)
-        self.assertLess(res['mase']['bottom'], 4)
+        pred, pred_scaled = self.gpf.predict(model, like)
+        res = self.gpf.metrics(pred[0], pred[1])
+        self.assertLess(res['mase']['bottom'], 5)
 
-    def test_results_interval_zero_noclip(self):
-        model, like = self.gpf.train(n_iterations=100, mean_function='zero')
-        samples = self.gpf.predict(model, like, False)
-        res = self.gpf.metrics(samples)
-        self.assertLess(res['mase']['bottom'], 4)
